library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- ----------------------------------------------------------------------------
-- Module:     StMemVar16
-- Purpose:    StMemVar16 is a known Place in Memory with status information.
--             It provides a 16bit interface for singleclock parallel access and
--             a 8bit interface for multiclock read and write access (i.e. for
--             picoblaze).      
-- Author:     myself
-- Created:    20090808
-- Behaviour:  The status tracks last access operation, so it is possible to 
--             synchronize access from different places.
--             clk is supposed to be fast system clock (i.e. the clock of the
--             picoblaze), where as the parallel access may happen at slower
--             clock.
--             status:  00 - means undef
--                      01 - last access was read access
--                      10 - last access was write access
-- ----------------------------------------------------------------------------
ENTITY StMemVar16 IS
    Port     ( clk      : in  std_logic;                       -- system clock
               we       : in  std_logic;                       -- write enable (active high)
               re       : in  std_logic;                       -- read enable  (active high)
               rd       : in  std_logic;                       -- read done (for 16bit interface)
               sdi      : in  std_logic_vector(7 downto 0);    -- 8-Bit interface input
               sdo      : out std_logic_vector(7 downto 0);    -- 8-Bit interface output
               pdi      : in  std_logic_vector(15 downto 0);   -- parallel interface input
               pdo      : out std_logic_vector(15 downto 0);   -- parallel interface output
               status   : out std_logic_vector(1 downto 0)
             );
END StMemVar16;

ARCHITECTURE std of StMemVar16 IS
   type reg_type is record
      fWrite : std_logic;
      fRead  : std_logic;
      od     : std_logic_vector(7 downto 0);
      data   : std_logic_vector(15 downto 0);
      tmp    : std_logic_vector(7 downto 0);
      status : std_logic_vector(1 downto 0);
   end record;
   constant STAT_UNDEF : std_logic_vector(1 downto 0) := "00";
   constant STAT_READ  : std_logic_vector(1 downto 0) := "01";
   constant STAT_WRITE : std_logic_vector(1 downto 0) := "10";
   constant rdef : reg_type := (
      fWrite => '1',
      fRead  => '1',
      od     => (others=>'0'),
      data   => (others=>'0'),
      tmp    => (others=>'0'),
      status => STAT_UNDEF 
   );
   signal r   : reg_type := rdef;
   signal rin : reg_type := rdef;
BEGIN

   comp: PROCESS(r, we, re, rd, pdi, sdi)
      variable v : reg_type;
   BEGIN
      v := r;

      if (we = '1') then
         if (r.fWrite = '1') then
            v.fWrite := '0';
            v.tmp    := sdi; 
         else
            v.fWrite := '1';
            v.data   := v.tmp & sdi;
            v.tmp    := (others=>'0');
            v.status := STAT_WRITE;
         end if;
      end if;
      if (re = '1') then
         if (r.fRead = '1') then
            v.fRead := '0';
            v.od    := r.data(15 downto 8);
         else   
            v.fRead  := '1';
            v.od     := r.data(7 downto 0);
            v.status := STAT_READ;
         end if;   
      end if;
      if (rd = '1') then
         v.status := STAT_READ;
      end if;
      if (pdi /= r.data) then
         v.data := pdi;
         v.status := STAT_WRITE;
      end if;
      sdo    <= r.od;
      pdo    <= r.data;
      status <= r.status;
      rin    <= v;
   END PROCESS;


   regs: PROCESS
   BEGIN
      wait until rising_edge(clk);
      r <= rin;
   END PROCESS;      
END std;
